% Main Program for Simulation in Cai, Fan, Jiang and Zhou (2004)
% Name: main_ex1_11_1.m
% 
% Programmed by: Jiancheng Jiang, UNC at Chapel Hill
% Updated date:  May 11, 2004
%-------------------------------------------------------------------------
close all;
global T Z X Delta W V vgrid h hopt;

nsim=10;             % Number of simulations, need to greater than one
n=100;                % Number of clusters
J=2;                 % the cluster size, must be greater than 1
theta=0.1;          % the parameter in the model of Clayton and Cuzick (1985), the smaller is the theta, the bigger is the correlation
beta=[0.8;0.6;0.4];      % the parameter \beta for the coef of W
p=length(beta);      % number of beta parameters
v0=linspace(0,1,20); % the grid points over which alpha() is estimated

vtype=1;
zparam=0.5;
ctype=1;             % uniform censoring
cparam=0.6;            % control ratio of censoring
stoptime=100000000;
hopt=0.2*n^(-1/5);     %  the bandwidth for estimating \alpha() and its derivatives
h=hopt*n^(-2/15);       %  the bandwidth for estimating \alpha() and \beta, using local lik

hatalpha1=zeros(nsim,length(v0));
hatalpha2=zeros(nsim,length(v0));
hatbeta=zeros(nsim,p);
hatomega=zeros(nsim,p);
hatnaomega=zeros(nsim,p);
coverage_90CI_new=zeros(nsim,p);
coverage_90CI_naive=zeros(nsim,p);
coverage_95CI_new=zeros(nsim,p);
coverage_95CI_naive=zeros(nsim,p);
coverage_99CI_new=zeros(nsim,p);
coverage_99CI_naive=zeros(nsim,p);
mse_g=zeros(nsim,1);
vargder=zeros(nsim,length(v0));
ratio_cens=zeros(nsim,1);
corr_T=zeros(nsim,J);


%------------------------ Simulations begin ------------------------
tic,t=cputime;
for i=1:nsim
randn('seed',i);
rand('seed',i);
[hatbeta0,hatalpha0]=main_temp(i,n,J,beta,theta,vtype,zparam,ctype,cparam,stoptime,v0,p,nsim);
hatbeta(i,:)=hatbeta0;              % Estimated \beta
hatalpha1(i,:)=hatalpha0(1,:)';     % Estimated alpha1(.) at the grid points v0
hatalpha2(i,:)=hatalpha0(2,:)';     % Estimated alpha1(.) at the grid points v0

%hatomega(i,:)=estomega';           % Estimated \Omega
%hatnaomega(i,:)=naomega';          % Estimated naive \Omega, the inverse of \hat{I}
%mycross(i,:)=cross_corr';          % Estimated cross corr ...
%hatalpha11=interp1(v0,hatalpha0(1,:),V,'linear');    % linear interpolation for alpha() at V, --a matrix n*J, naive estimator of alpha()
%hatalpha21=interp1(v0,hatalpha0(2,:),V,'linear');    % linear interpolation for alpha() at V, --a matrix n*J, naive estimator of alpha()

mse_alpha1(i)=mean(mean((hatalpha0(1,:)-falpha1(v0)).^2));
mse_alpha2(i)=mean(mean((hatalpha0(2,:)-falpha2(v0)).^2));

%estsig=sqrt(estomega'/n);
%estna=sqrt(naomega'/n);
%coverage_90CI_new(i,:)=(beta'<=hatbeta0+1.6449*estsig).*(beta'>=hatbeta0-1.6449*estsig); % using N(0,1)'s 0.05th and 0.95th percentiles
%coverage_90CI_naive(i,:)=(beta'<=hatbeta0+1.6449*estna).*(beta'>=hatbeta0-1.6449*estna); 
%coverage_95CI_new(i,:)=(beta'<=hatbeta0+1.96*estsig).*(beta'>=hatbeta0-1.96*estsig);     % using N(0,1)'s 0.025th and 0.975th percentiles
%coverage_95CI_naive(i,:)=(beta'<=hatbeta0+1.96*estna).*(beta'>=hatbeta0-1.96*estna); 
%coverage_99CI_new(i,:)=(beta'<=hatbeta0+2.5758*estsig).*(beta'>=hatbeta0-2.5758*estsig); % using N(0,1)'s 0.005th and 0.995th percentiles
%coverage_99CI_naive(i,:)=(beta'<=hatbeta0+2.5758*estna).*(beta'>=hatbeta0-2.5758*estna); 
%for j=1:length(z0)
%    vargder(i,j)=var_gder(hatbeta0,gz1,z0(j),hopt);
%end
ratio_cens(i)=mean(mean(Delta));
corr=corrcoef(T);
corr_T(i,:)=corr(:,1)';
end;
cputime-t
% ---------------  Simulations end --------------------------------------
%vargder=sqrt(mean(vargder));
mhatalpha1=mean(hatalpha1);
mhatalpha2=mean(hatalpha2);

%CI_UP_gder=mgderiv+1.96*vargder;
%CI_LOW_gder=mgderiv-1.96*vargder; 
mhatbeta=mean(hatbeta,1)   % estimated mean of beta
stdbeta=std(hatbeta,1)     % standard deviation of \hat{\beta}
alpha1975=quantile(hatalpha1,0.975);
alpha1025=quantile(hatalpha1,0.025);
alpha150=quantile(hatalpha1,0.50);
alpha2975=quantile(hatalpha2,0.975);
alpha2025=quantile(hatalpha2,0.025);
alpha250=quantile(hatalpha2,0.50);

%eststdbeta=sqrt(mean(hatomega/n,1));      % average of estimated standard error 
%nastdbeta=sqrt(mean(hatnaomega/n,1));     % naive approach: average of estimated error
%stdeststdbeta=std(sqrt(hatomega/n),1);    % std of estimated standard error 
%stdnastdbeta=std(sqrt(hatnaomega/n),1);   % naive approach: std of estimated error
ratiocen=1-mean(ratio_cens);            % ratio of censoring
b=corr_T(:,2);
corr_Ti=mean(b(abs(b)>0));
%----------------  report results   ---------------------------------------
%myresults_beta=cat(2,mhatbeta',stdbeta',eststdbeta',nastdbeta',stdeststdbeta',stdnastdbeta');
%myresults_g=cat(2,h,ave_mse_g,med_mse_g,std_mse_g);
%display('Results reported:=[mean,std,estimated std, naive estimated std,std of estimated std, std of naive estimated std of the estimators for beta]');
%myresults_beta
%display('Results reported:=[h; ave,med,std of MSE of the estimators for g] ');
%myresults_g
%display('90% coverage of CI for beta:=[the proposed method,naive method]');
%[mean(coverage_90CI_new)',mean(coverage_90CI_naive)']
%display('95% coverage of CI for beta:=[the proposed method,naive method]');
%[mean(coverage_95CI_new)',mean(coverage_95CI_naive)']
%display('99% coverage of CI for beta:=[the proposed method,naive method]');
%[mean(coverage_99CI_new)',mean(coverage_99CI_naive)']

display('Ratio of censoring:=');
ratiocen
display('theta');
theta
display('Corrcoef of T');
corr_Ti
%display('Cross Correlation');
%mean(mycross)
% save key results ----------------------------------
%save ex1_1_hatbeta.dat  hatbeta /ascii;  % save the results
%save ex1_1_hatg.dat hatg /ascii;
%save ex1_1_mseg.dat mse_g /ascii;
%save ex1_1_hatomega.dat hatomega /ascii;
%save ex1_1_hatnaomega.dat hatnaomega /ascii;

%--------------- plots --------------------------
[alpha1,alpha1d,alpha1d2]=falpha1(v0);
[alpha2,alpha2d,alpha2d2]=falpha2(v0);

pstr='ex1_1';
figure(1);%close  % to use the matlab Batch mode on unix to yield graphs
clf;
plot(v0,mhatalpha1,'k--');
line(v0,alpha1);
set(gca,'fontsize',18);
axis([-0.1,1.1,min(alpha1)-abs(min(alpha1))*0.5-2,max(alpha1)+0.5*max(alpha1)+2]);
hold on
plot(v0,alpha1975,'k-.');
hold on
plot(v0,alpha1025,'k-.');
hold on 
plot(v0,alpha150,'r:');
title('Estimated function \alpha_1(\cdot) with percentiles');
orient portrait;
print('-deps', [pstr,'.eps']);


pstr='ex1_2';
figure(2);%close  % to use the matlab Batch mode on unix to yield graphs
clf;
plot(v0,mhatalpha2,'k--');
line(v0,alpha2);
set(gca,'fontsize',18);
axis([-0.1,1.1,min(alpha2)-abs(min(alpha2))*0.5-2,max(alpha2)+0.5*max(alpha2)+2]);
hold on
plot(v0,alpha2975,'k-.');
hold on
plot(v0,alpha2025,'k-.');
hold on 
plot(v0,alpha250,'r:');
title('Estimated function \alpha_2(\cdot) with percentiles');
orient portrait;
print('-deps', [pstr,'.eps']);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%% Performance of the estimation: quantiles based on ranking MISEs %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
[MSEg1,I]=sort(mse_alpha1);
Li=length(I);
index_10=I(ceil(0.1*Li));
index_50=I(ceil(0.5*Li));
index_90=I(ceil(0.9*Li));
index=[index_10,index_50,index_90];
for i=1:3
randn('seed',index(i));
rand('seed',index(i));
[hatbeta0,hatalpha0]=main_temp(i,n,J,beta,theta,vtype,zparam,ctype,cparam,stoptime,v0,p,nsim);
hatalpha1_typical(i,:)=hatalpha0(1,:)';                      % Typical Estimated alpha_1(.) at the grid points z0
end;

figure(3);close  % to use the matlab Batch mode on unix to yield graphs
clf;
plot(v0,hatalpha1_typical(2,:),'k--','LineWidth',1);
line(v0,alpha1);
hold on
plot(v0,hatalpha1_typical(1,:),'k-.','LineWidth',1);
hold on
plot(v0,hatalpha1_typical(3,:),'k:','LineWidth',2.2);
set(gca,'fontsize',24);
axis([-0.140,1.1,min(alpha1)-abs(min(alpha1))*0.5-1,max(alpha1)+0.5*max(alpha1)+2]);
%xlabel('(a)','FontSize',28);
title('\theta =0.01, 40% censoring');
orient portrait;
print('-deps', [pstr,'_typical_1.eps']);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%% Performance of the estimation: quantiles based on ranking MISEs %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
[MSEg1,I]=sort(mse_alpha2);
Li=length(I);
index_10=I(ceil(0.1*Li));
index_50=I(ceil(0.5*Li));
index_90=I(ceil(0.9*Li));
index=[index_10,index_50,index_90];
for i=1:3
randn('seed',index(i));
rand('seed',index(i));
[hatbeta0,hatalpha0]=main_temp(i,n,J,beta,theta,vtype,zparam,ctype,cparam,stoptime,v0,p,nsim);
hatalpha2_typical(i,:)=hatalpha0(2,:)';                      % Typical Estimated alpha_1(.) at the grid points z0
end;

figure(4);close  % to use the matlab Batch mode on unix to yield graphs
clf;
plot(v0,hatalpha2_typical(2,:),'k--','LineWidth',1);
line(v0,alpha2);
hold on
plot(v0,hatalpha2_typical(1,:),'k-.','LineWidth',1);
hold on
plot(v0,hatalpha2_typical(3,:),'k:','LineWidth',2.2);
set(gca,'fontsize',24);
axis([-0.140,1.1,min(alpha2)-abs(min(alpha2))*0.5-1,max(alpha2)+0.5*max(alpha2)+2]);
%xlabel('(a)','FontSize',28);
title('\theta =0.01, 40% censoring');
orient portrait;
print('-deps', [pstr,'_typical_2.eps']);






