% Main Program for Simulation in Cai, Fan, Jiang and Zhou (2004)
% Name: ex2_01.m
% 
% Programmed by: Jiancheng Jiang, UNC at Chapel Hill
% Updated date:  May 11, 2004
%-------------------------------------------------------------------------
close all;
global T Z X Delta W V vgrid h hopt p alphaV;

nsim=500;             % Number of simulations, need to greater than one
n=100;                % Number of clusters
J=2;                 % the cluster size, must be greater than 1
theta=0.01;          % the parameter in the model of Clayton and Cuzick (1985), the smaller is the theta, the bigger is the correlation
beta=[0.8;0.6;1.0];      % the parameter \beta for the coef of W
p=length(beta);      % number of beta parameters
v0=linspace(0,1,20); % the grid points over which alpha() is estimated

vtype=1;
zparam=0.5;
ctype=1;             % uniform censoring
cparam=0.6;            % control ratio of censoring
stoptime=100000000;

hatalpha=zeros(nsim,length(v0));
hatbeta=zeros(nsim,p);
hatomega=zeros(nsim,p);
hatnaomega=zeros(nsim,p);
coverage_90CI_new=zeros(nsim,p);
coverage_90CI_naive=zeros(nsim,p);
coverage_95CI_new=zeros(nsim,p);
coverage_95CI_naive=zeros(nsim,p);
coverage_99CI_new=zeros(nsim,p);
coverage_99CI_naive=zeros(nsim,p);
mse_g=zeros(nsim,1);
vargder=zeros(nsim,length(v0));
ratio_cens=zeros(nsim,1);
corr_T=zeros(nsim,J);

simul_data_T=zeros(n,J,nsim);
simul_data_X=zeros(n,J,nsim);
simul_data_V=zeros(n,J,nsim);
simul_data_Z=zeros(n,J,nsim);
simul_data_W=zeros(n,J,p,nsim);
simul_data_Delta=zeros(n,J,nsim);
mycross=zeros(nsim,p);

hopt=0.5*n^(-1/5);     %  the bandwidth for estimating \alpha() and its derivatives
h=hopt*n^(-2/15);       %  the bandwidth for estimating \alpha() and \beta, using local lik

%------------------------ Simulations begin ------------------------
tic,t=cputime;
for i=1:nsim
randn('seed',i);
rand('seed',i);

[hatbeta0,hatalpha0,estomega,naomega,cross_corr]=main_temp2(i,n,J,beta,theta,vtype,zparam,ctype,cparam,stoptime,v0,nsim);
hatbeta(i,:)=hatbeta0;              % Estimated \beta
hatalpha(i,:)=hatalpha0;            % Estimated alpha(.) at the grid points v0

hatomega(i,:)=estomega';           % Estimated \Omega
hatnaomega(i,:)=naomega';          % Estimated naive \Omega, the inverse of \hat{I}
mycross(i,:)=cross_corr';          % Estimated cross corr ...

mse_alpha(i)=mean(mean((hatalpha0(1,:)-falpha1(v0)).^2));

estsig=sqrt(estomega'/n);
estna=sqrt(naomega'/n);
coverage_90CI_new(i,:)=(beta'<=hatbeta0+1.6449*estsig).*(beta'>=hatbeta0-1.6449*estsig); % using N(0,1)'s 0.05th and 0.95th percentiles
coverage_90CI_naive(i,:)=(beta'<=hatbeta0+1.6449*estna).*(beta'>=hatbeta0-1.6449*estna); 
coverage_95CI_new(i,:)=(beta'<=hatbeta0+1.96*estsig).*(beta'>=hatbeta0-1.96*estsig);     % using N(0,1)'s 0.025th and 0.975th percentiles
coverage_95CI_naive(i,:)=(beta'<=hatbeta0+1.96*estna).*(beta'>=hatbeta0-1.96*estna); 
coverage_99CI_new(i,:)=(beta'<=hatbeta0+2.5758*estsig).*(beta'>=hatbeta0-2.5758*estsig); % using N(0,1)'s 0.005th and 0.995th percentiles
coverage_99CI_naive(i,:)=(beta'<=hatbeta0+2.5758*estna).*(beta'>=hatbeta0-2.5758*estna); 
%for j=1:length(z0)
%    vargder(i,j)=var_gder(hatbeta0,gz1,z0(j),hopt);
%end
ratio_cens(i)=mean(mean(Delta));
corr=corrcoef(T);
corr_T(i,:)=corr(:,1)';

simul_data_T(:,:,i)=T;  % keep the data for later analysis
simul_data_X(:,:,i)=X;
simul_data_V(:,:,i)=V;
simul_data_Z(:,:,i)=Z;
simul_data_Delta(:,:,i)=Delta;
simul_data_W(:,:,:,i)=W;
end;
cputime-t

save ex3_simul_data01 simul_data_T simul_data_X simul_data_Delta simul_data_W simul_data_V simul_data_Z; 
% save the variable for later use in matlab format

% ---------------  Simulations end --------------------------------------
%vargder=sqrt(mean(vargder));
mhatalpha=mean(hatalpha);

%CI_UP_gder=mgderiv+1.96*vargder;
%CI_LOW_gder=mgderiv-1.96*vargder; 

mhatbeta=mean(hatbeta,1)   % estimated mean of beta
stdbeta=std(hatbeta,1)     % standard deviation of \hat{\beta}
alpha975=quantile(hatalpha,0.975);
alpha025=quantile(hatalpha,0.025);
alpha50=quantile(hatalpha,0.50);

eststdbeta=sqrt(mean(hatomega/n,1));      % average of estimated standard error 
nastdbeta=sqrt(mean(hatnaomega/n,1));     % naive approach: average of estimated error
stdeststdbeta=std(sqrt(hatomega/n),1);    % std of estimated standard error 
stdnastdbeta=std(sqrt(hatnaomega/n),1);   % naive approach: std of estimated error
ratiocen=1-mean(ratio_cens);            % ratio of censoring
b=corr_T(:,2);
corr_Ti=mean(b(abs(b)>0));
%----------------  report results   ---------------------------------------
myresults_beta=cat(2,mhatbeta',stdbeta',eststdbeta',nastdbeta',stdeststdbeta',stdnastdbeta');
display('Results reported:=[mean,std,estimated std, naive estimated std,std of estimated std, std of naive estimated std of the estimators for beta]');
myresults_beta
display('90% coverage of CI for beta:=[the proposed method,naive method]');
[mean(coverage_90CI_new)',mean(coverage_90CI_naive)']
display('95% coverage of CI for beta:=[the proposed method,naive method]');
[mean(coverage_95CI_new)',mean(coverage_95CI_naive)']
display('99% coverage of CI for beta:=[the proposed method,naive method]');
[mean(coverage_99CI_new)',mean(coverage_99CI_naive)']

display('Ratio of censoring:=');
ratiocen
display('theta');
theta
display('Corrcoef of T');
corr_Ti
display('Cross Correlation');
mean(mycross)
% save key results ----------------------------------
%save ex1_1_hatbeta.dat  hatbeta /ascii;  % save the results
%save ex1_1_hatg.dat hatg /ascii;
%save ex1_1_mseg.dat mse_g /ascii;
%save ex1_1_hatomega.dat hatomega /ascii;
%save ex1_1_hatnaomega.dat hatnaomega /ascii;

%--------------- plots --------------------------
[alpha,alphad,alphad2]=falpha1(v0);

pstr='ex3_01';
figure(1);close  % to use the matlab Batch mode on unix to yield graphs
clf;
plot(v0,mhatalpha,'k--');
line(v0,alpha);
set(gca,'fontsize',22);
axis([-0.1,1.1,min(alpha)-abs(min(alpha))*0.5-2,max(alpha)+0.5*max(alpha)+2]);
hold on
plot(v0,alpha975,'k-.');
hold on
plot(v0,alpha025,'k-.');
hold on 
plot(v0,alpha50,'r:');
title('Estimated function \alpha(\cdot) with percentiles');
orient portrait;
print('-deps', [pstr,'.eps']);


%%%%%%%%%%%%%%%%%%%%%%%%%%%%% Performance of the estimation: quantiles based on ranking MISEs %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
[MSEg1,I]=sort(mse_alpha);
Li=length(I);
index_10=I(ceil(0.1*Li));
index_50=I(ceil(0.5*Li));
index_90=I(ceil(0.9*Li));
index=[index_10,index_50,index_90];
mm=length(index);
hatalpha_typical=zeros(mm,length(v0));
for i=1:mm
randn('seed',index(i));
rand('seed',index(i));
[hatbeta0,hatalpha0]=main_temp1(i,n,J,beta,theta,vtype,zparam,ctype,cparam,stoptime,v0,nsim);
hatalpha_typical(i,:)=hatalpha0;                      % Typical Estimated alpha_1(.) at the grid points z0
end;

figure(2);close  % to use the matlab Batch mode on unix to yield graphs
clf;
plot(v0,hatalpha_typical(2,:),'k--','LineWidth',1);
line(v0,alpha);
hold on
plot(v0,hatalpha_typical(1,:),'k-.','LineWidth',1);
hold on
plot(v0,hatalpha_typical(3,:),'k:','LineWidth',2.2);
set(gca,'fontsize',22);
axis([-0.140,1.1,min(alpha)-abs(min(alpha))*0.5-1,max(alpha)+0.5*max(alpha)+2]);
title('Typical estimated curves');
orient portrait;
print('-deps', [pstr,'_typical.eps']);


