% Computing logarithm of the pseudo-partial likelihood function in (6), given beta
% Name: global_ell.m
% Input: global Z,X,Delta,W,gridp,h; local beta,gamma
%        Z-- covariate matrix (n*J);
%        X-- min(T,C), the observed time, a matrix (n*J)
%        Delta-- censoring index, a matrix (n*J)
%        W-- covariate array (n*J*p), where p is the dimension of beta
%        beta-- coef vector for W;   gamma- approx for g(.), parameter
%        gridp-- grid points for z_0, a scalar here
%        h- the bandwidth, a scalar for global bandwidth
% Output: the minus logarithm of the global pseudo partial likelihood evaluated at beta
% Functions called: 
%
% Programmed by: Jiancheng Jiang, Department of Biostatistics, UNC-CH
% Date: May 1, 2004

function [ell]=global_ell(beta)
global Z X V Delta W alphaV p;
[n,J]=size(X);
    Wb=0;
    for i=1:p
       Wb=Wb+W(:,:,i)*beta(i);    % find $beta^T W_{ij}$
   end
ell=Wb+alphaV;             % find $\beta^T W_{ij}+\gamma(Z_{ij})$

a=zeros(1,J,n);
a(1,:,:)=X';
Rj=(repmat(a,[n 1 1])>=repmat(X,[1 1 n]));
a=zeros(1,J,n);
a(1,:,:)=exp(ell)';
inlog=sum(repmat(a,[n 1 1]).*Rj,3);
%inlog=zeros(n,J);
%for i=1:n;
%Rji=(X>=repmat(X(i,:),n,1));    % Rj is an array with (i,j,l)th element (X_{lj}>=X_{ij}), i=1,...,n; l=1,...,n;j=1,...,J
%inlog(i,:)=sum(exp(ell).*Rji);  % the sum in log() in (5)
%end

ell=ell-log(inlog);
ell=-sum(sum(ell.*Delta));
return




