% Computing logarithm of the local pseudo-partial likelihood function in (5) at zgrid points, given alpha=(beta',gamma)
% Name: loc_ell.m
% Input: global Z,X,Delta,W,gridp,h; local beta,gamma
%        Z-- covariate matrix (n*J);
%        X-- min(T,C), the observed time, a matrix (n*J)
%        Delta-- censoring index, a matrix (n*J)
%        W-- covariate array (n*J*p), where p is the dimension of beta
%        alpha=(beta',gamma')'; beta-- coef vector for W;   gamma- approx for g(.), parameter
%        zgrid-- grid point, z_0, a scalar
%        h- the bandwidth, a scalar for global bandwidth
% Output: the minus local pseudo partial likelihood evaluated at alpha and z_0
% Functions called: epan.m, or my_gauss.m, the kernel function
%
% Programmed by: Jiancheng Jiang, Department of Biostatistics, UNC-CH
% Date: April 15, 2004

function [ell]=loc_ell(alpha)
global Z X V Delta W vgrid h p;
beta=alpha(1:p);
eta=alpha((p+1):end);
[n,J]=size(X);
    Wb=0;
    for i=1:p
       Wb=Wb+W(:,:,i)*beta(i);      % find $beta^T W_{ij}$, a single n*J matrix
    end
v=V-vgrid;

ell=Wb+eta(1)*Z+eta(2)*Z.*v;     
% find $\beta^T W_{ij}+\gamma_1Z_{ij}+\gamma_2 Z_{ij}*(V_{ij}-vgrid)$, an array of dimension n*J
kh=1/h*epan(v/h);

a=zeros(1,J,n);
a(1,:,:)=X';
Rj=(repmat(a,[n 1 1])>=repmat(X,[1 1 n]));
a=zeros(1,J,n);
a(1,:,:)=exp(ell)'.*kh';
inlog=sum(repmat(a,[n 1 1]).*Rj,3);  % using no-loop replace the following loop, equivalently
%inlog=zeros(n,J);
%for i=1:n;
%Rji=(X>=repmat(X(i,:),n,1));        % Rji is an matrix with (l,j)th element (X_{lj}>=X_{ij}), for i fixed
%inlog(i,:)=sum(exp(ell).*kh.*Rji);  % the sum in log() in (5)
%end

ell=ell-log(inlog);                 % an array of dimension n*J
ell=-sum(sum(ell.*Delta.*kh));
return
 



