function [mhat,mhatder]=loc_linear_der(y,x,h,ingrid)
% all vectors are column vector if not declared
% Local Linear Regressionwith given local bandwidths at ingrid   
% Model: y_i=m(x_i)+e_i
% Name:  loc_linear.m
% input: y-a vector,x-a vector,ingrid-a vector for estimation grid points, 
%        h-a scalar
% function called: kx.m
% output: mhat, the estimated value of m(x) at ingrid, a column vector;
% \hat{m}', the derivative of m(x)
% designed by Jiancheng Jiang, July 01, 2001 at CUHK

n=length(y);                          % samples size
m=length(ingrid);                     % number of grid points 
xz=repmat(x,1,m)-repmat(ingrid',n,1); % a n*m matrix
kxxizh=epan(xz/h);     % a n*m matrix

sn1=sum(kxxizh.*xz);                  % a row vector: 1*m
sn2=sum(kxxizh.*xz.^2);               % a row vector: 1*m
sn0=sum(kxxizh);

sn0=sn0+h'*0.2;   % ridge parameter for epan kernel to deal with the possibility of zero denominate.
sn2=sn2+h'*0.2;   % ridge parameter for epan kernel to deal with the possibility of zero denominate. 

wj=kxxizh.*(repmat(sn2,n,1)-xz.*repmat(sn1,n,1));
mhat=sum(wj.*repmat(y,1,m))./(sn0.*sn2-sn1.^2); % a column vector----\hat{m}
wj=kxxizh.*(xz.*repmat(sn0,n,1)-repmat(sn1,n,1));
mhatder=sum(wj.*repmat(y,1,m))./(sn0.*sn2-sn1.^2); % a column vector----\hat{m}'
return
