%  Name: local_clayton.m; For generating random number of survival times by Claton model
%  Input: J--the number of obs in the ith cluster, i=1,...,n
%         n------the number of subjects/cluster
%         Z------covariate z, a column vector (n*J); W--covariate, an array, three dimensions, (n*J*p) 
%         beta---the coef of W, a column p-vector; theta-- the parameter in the joint survival function
%  Output: the random numbers of $T_{ij}$, i=1,...,n;j=1,...,J
%  Functions called: St(t), the marginal survival function for $T_i$ in simulations; here there is no need to input by Cai and Shen (2000)
%                    gt(t), the nonparametric function g(.) for Z
%  Programmed by Jiancheng Jiang, April 15, 2004, UNC-CH

function tt=local_clayton(n,J,z,w,v,theta,beta)
u=unifrnd(0,1,n,J);
tt=zeros(n,J);
lambda=(1:J).^2;
suma=zeros(n,1);
  Wb=0;
  p=length(beta);
   for i=1:p
       Wb=Wb+w(:,:,i)*beta(i);    % find $beta^T W_{ij}$
   end
   
   [alpha1,alpha1d,alpha2d2]=falpha1(v);
   [alpha2,alpha2d,alpha2d2]=falpha2(v);

   Wb=Wb+alpha1.*z(:,:,1)+alpha2.*z(:,:,2);                   % the alpha(V_{ij})^T Z_{ij})+\beta'W_{ij}, a n*J  matrix 

tt(:,1)=-log(1-u(:,1)).*exp(-Wb(:,1))/lambda(1);
if (J>1)
	for i=2:J
	suma=suma+exp(tt(:,i-1).*exp(Wb(:,i-1))/theta*lambda(i-1));
	b1=log((i-1)-suma+(suma-(i-2)).*(1-u(:,i)).^(-(theta+i-1)^(-1)));
	tt(:,i)=theta.*b1.*exp(-Wb(:,i))/lambda(i);
    end
end
return	