function [estomega,naomega,cross_corr]=omega(beta,alphav)
% function: find estimation of the covariance matrix \Omega

global Z X Delta W V p alphaV;
[n,J]=size(X);
p=length(beta);
v0=linspace(0,1,20)';
q=length(v0);
h=zeros(q,J);

for j=1:J
 ss=std(V(:,j));
 h(:,j)=1.06*ss*n^(-0.2)*ones(q,1);
end

Yijm=(repmat(X,[1,1,n])>=permute(repmat(X,[1 1 n]),[3,2,1]));  % n*J*n array: the (i,j,m) element is Y_{ij}(X_{mj})
Wb=0;
    for i=1:p
       Wb=Wb+W(:,:,i)*beta(i);      % find $beta^T W_{ij}$, a single n*J matrix
    end
r=repmat(exp(Wb+alphav),[1 1 n]);       
Yijmr=Yijm.*r;
rj0=mean(Yijmr,1);  
rj0=squeeze(rj0)'; % the (m,j) element of r_{j0}(X_{mj}), a matrix of n*J
rj1=zeros(n,J,p);  % 
[est_chiv]=chiv(v0,Yijmr,p,rj0,h); % a q*p matrix
mychiv=zeros(n,J,p);
hatX=zeros(n,J,p);

for s=1:p
	mychiv(:,:,s)=interp1(v0,est_chiv(:,s),V);
    hatX(:,:,s)=W(:,:,s)+mychiv(:,:,s).*Z;
	W1=repmat(hatX(:,:,s),[1 1 n]);
    W11=mean(Yijmr.*W1,1);  
    rj1(:,:,s)=squeeze(W11)'; % the sth component of r_{j1}(X_{mj}), a matrix of n*J whose element is r_{j1}'s the sth component (m,j)
end
rj10=rj1./repmat(rj0,[1 1 p]); % rj1./rj0, the (m,j,s) element

%%%%%%%%%%%%%  estimating \hat{s}(Z_{ij})  %%%%%%%%%%%%%%%%%%%%%%%
hatsv=zeros(n,J,p);              % to find \hat{s}(V_{ij}), n*J*p array
hatsv0=hats(v0,Yijmr,p,rj0,rj10,hatX,h);  % q*p matrix
for s=1:p
    for j=1:J
    hatsv(:,j,s)=interp1(v0,hatsv0(:,s),V(:,j));
    end
end
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
estrhoj0=rhoj0(v0,Yijmr,h);
estrhoj1=rhoj1(v0,Yijmr,h);
estrhoj2=rhoj2(v0,Yijmr,h);
estrhoj10=estrhoj1./estrhoj0;
for j=1:J
    for m=1:n
        cketa(m,j,:)=interp1(v0,squeeze(estrhoj10(m,j,:)),V(:,j));
    end
end
cketa1=zeros(n,J);
for j=1:J
    cketa1(:,j)=diag(squeeze(cketa(:,j,:)));
end
czsig1=hatsv.*repmat((Z-cketa1),[1 1 p]);  % the (i,j,s) element
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
Gij=zeros(n,J,p);
W0=permute(repmat(Delta,[1 1 n]),[ 3 2 1]).*permute(Yijmr,[3 2 1])./repmat(rj0,[1 1 n]); % the (m,j,i) element
for s=1:p
    czsig2=repmat(hatsv(:,:,s),[1 1 n]).*(repmat(Z,[1 1 n])-permute(cketa,[3 2 1]));  % the (i,j,m) element
    czsig2=permute(czsig2,[3 2 1]);  % the (m,j,i) element
    W2=permute(repmat(hatX(:,:,s),[1 1 n]),[3 2 1])-repmat(rj10(:,:,s),[1 1 n]);  % the (m,j,i) element
    W2=W2-czsig2;
    W1=mean(W0.*W2,1);
    W1=squeeze(W1)';  % a matrix of n*J whose element is the (i,j) element of the second term of \hat{G}_{ij}
    Gij(:,:,s)=(hatX(:,:,s)-rj10(:,:,s)-czsig1(:,:,s)).*Delta-W1;  % G_{ij}'s the s-th component, n*J matrix
end
estsigma=zeros(p,p);
rj2=zeros(n,J,p,p);
I=zeros(p,p);
for s=1:p
    W1=repmat(hatX(:,:,s),[1 1 n]);
    for l=1:p
    W2=repmat(hatX(:,:,l),[1 1 n]);
    rj2(:,:,s,l)=squeeze(mean(Yijmr.*W1.*W2,1))';
    I(s,l)=sum(mean(Delta.*(rj2(:,:,s,l)./rj0-rj10(:,:,s).*rj10(:,:,l)),1));
    I1(s,l)=mean(sum(Gij(:,:,s).*Gij(:,:,l),2));
    W0=sum(Gij(:,:,s),2).*sum(Gij(:,:,l),2);
    W0=mean(W0);
    estsigma(s,l)=W0;   % \hat{\Sigma}_{sl}
    end
end
cross_corr=diag(estsigma-I1);
naomega=inv(I);
estomega=diag(naomega*estsigma*naomega');  % \hat{\Omega}_{ii}
naomega=diag(naomega);                    % naive estimated \Omega_{ii}
return

