function [estomega,naomega,cross_corr]=omega(beta,gz)
% function: find estimation of the covariance matrix \Omega

global Z X Delta W;
[n,J]=size(X);
p=length(beta);
Yijm=(repmat(X,[1,1,n])>=permute(repmat(X,[1 1 n]),[3,2,1]));  % n*J*n array: the (i,j,m) element is Y_{ij}(X_{mj})
Wb=0;
    for i=1:p
       Wb=Wb+W(:,:,i)*beta(i);      % find $beta^T W_{ij}$, a single n*J matrix
    end
r=repmat(exp(Wb+gz),[1 1 n]);       
Yijmr=Yijm.*r;
rj0=mean(Yijmr,1);  
rj0=squeeze(rj0)'; % the (m,j) element of r_{j0}(X_{mj}), a matrix of n*J
rj1=zeros(n,J,p);  % 
for s=1:p
    W1=repmat(W(:,:,s),[1 1 n]);
    W11=mean(Yijmr.*W1,1);  
    rj1(:,:,s)=squeeze(W11)'; % the sth component of r_{j1}(X_{mj}), a matrix of n*J whose element is r_{j1}'s the sth component (m,j)
end
rj10=rj1./repmat(rj0,[1 1 p]); % rj1./rj0, the (m,j,s) element
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
cz=zeros(n,J,p);  % to find \hat{c}(z)
Drj0=repmat(Delta./rj0,[1 1 p]); % n*J*p array
for j=1:J
    for i=1:n
        z=Z(i,j);
        cz0=rhom(z,Yijmr,rj10,p); % \hat{\rho}_j(X_{ij},z), n*J* array
        cz0=cz0.*Drj0;
        cz0=mean(cz0,1);
        cz0=squeeze(cz0);
        cz(i,j,:)=sum(cz0,1);
    end
end
sigmader=est_sigmader(Z);
czsig=cz.*repmat(sigmader,[1 1 p]);
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
Gij=zeros(n,J,p);
W0=permute(repmat(Delta,[1 1 n]),[ 3 2 1]).*permute(Yijmr,[3 2 1])./repmat(rj0,[1 1 n]); % the (m,j,i) element
for s=1:p
    W2=permute(repmat(W(:,:,s),[1 1 n]),[3 2 1])-repmat(rj10(:,:,s),[1 1 n]);  % the (m,j,i) element
    W2=W2-permute(repmat(czsig(:,:,s),[1 1 n]),[3 2 1]);
    W1=mean(W0.*W2,1);
    W1=squeeze(W1)';  % a matrix of n*J whose element is the (i,j) element of the second term of \hat{G}_{ij}
    Gij(:,:,s)=(W(:,:,s)-rj10(:,:,s)-czsig(:,:,s)).*Delta-W1;  % G_{ij}'s the s-th component, n*J matrix
end
estsigma=zeros(p,p);
rj2=zeros(n,J,p,p);
I=zeros(p,p);
for s=1:p
    W1=repmat(W(:,:,s),[1 1 n]);
    for l=1:p
    W2=repmat(W(:,:,l),[1 1 n]);
    rj2(:,:,s,l)=squeeze(mean(Yijmr.*W1.*W2,1))';
    I(s,l)=sum(mean(Delta.*(rj2(:,:,s,l)./rj0-rj10(:,:,s).*rj10(:,:,l)),1));
    %I(s,l)=mean(sum(Gij(:,:,s).*Gij(:,:,l),2));
    W0=sum(Gij(:,:,s),2).*sum(Gij(:,:,l),2);
    W0=mean(W0);
    estsigma(s,l)=W0;   % \hat{\Sigma}_{sl}
    end
end
cross_corr=diag(estsigma-I);
naomega=inv(I);
estomega=diag(naomega*estsigma*naomega');  % \hat{\Omega}_{ii}
naomega=diag(naomega);                    % naive estimated \Omega_{ii}
return

