% Computing integral of the derivative of a function over a grid points, 
%   $\hat{g}(z)=\int_{z_0}^{z_m} g'(t)dt$, given g'(z_i) and g(z_0)=0
% Trapezoidal Rule for Computing \hat{g} over Grid Points z=(z_0,z_1,...,z_m)', given the values of g'(z_j) and g(z_0)=0
% Name: trap.m
% Input: z -- grid points over which g is to be estimated, a row vector
%        gderiv -- the derivative of g(.) over z, a row vector
% Output: gz -- the estimated values of g(.) at z
% Programmed by Jiancheng Jiang, UNC-CH, May 1, 2004
% All vectors in column
%
% Example: z0=linspace(0,1,2); gderiv=cos(z0);g=trap(z0,gderiv);

function gz=trap(z,gderiv)
m=length(z);
z1=tril(ones(m-1,1)*z(2:end)');
z2=tril(ones(m-1,1)*z(1:(end-1))');
zz=z1-z2;
gz=0.5*zz*(gderiv(2:end)+gderiv(1:(end-1)));
gz=[0 gz'];
return
